@import url("alabaster.css");
@import url('//fonts.googleapis.com/css?family=Overpass:300,800|Roboto+Mono');

li {
    margin-top: 4px;
    margin-bottom: 4px;
}

p.caption {
    font-size: 90%;
}

div.body h1 { margin-bottom: 0.8em; font-size: 210%; }
div.body h2 { font-size: 160%; }
div.body h3 { font-size: 130%; }
div.body h4 { font-size: 125%; }
div.body h5 { font-size: 100%; }
div.body h6 { font-size: 100%; }

div.body h1, div.body h2, div.body h3,
div.body h4, div.body h5, div.body h6,
div.sphinxsidebar h1, div.sphinxsidebar h2,
div.sphinxsidebar h3, div.sphinxsidebar h4,
div.sphinxsidebar h5, div.sphinxsidebar h6 {
    font-family: 'Overpass', sans-serif;
    font-weight: 800;
    line-height: 1.1em;
}

div.body h1,
div.body h2 {
    color: #000;
}

div.sphinxsidebar h3 {
    font-size: 16px;
}

div.sphinxsidebar input {
    box-sizing: border-box;
    font-family: 'Overpass', sans-serif;
    font-weight: 200;
    line-height: 1.75;
}

div.sphinxsidebar form {
    display: flex;
}

div.sphinxsidebar #searchbox {
    margin-top: 1.5em;
}

div.sphinxsidebar #searchbox input[type="text"] {
    padding-top: 0;
    padding-bottom: 0;
    border-right: 0;
    border-radius: 4px 0 0 4px;
}

div.sphinxsidebar #searchbox input[type="submit"] {
    padding-top: 0;
    padding-bottom: 0;
    border-radius: 0 4px 4px 0;
}

div.sphinxsidebar ul {
    color: #999;
}

div.sphinxsidebar ul li.toctree-l2 {
    margin-top: 0px;
    margin-bottom: 0px;
}

div.sphinxsidebar ul li.toctree-l2 > a {
    font-size: 100%;
    color: #666;
    border-bottom: none;
    margin-top: 0px;
    margin-bottom: 0px;
}

div.sphinxsidebar ul li.toctree-l1 > a {
    line-height: 1.8em;
    font-size: 110%;
    font-weight: 800;
    border-bottom: none;
    margin-top: 0px;
    margin-bottom: 0px;
}

div.sphinxsidebarwrapper {
    padding-top: 0;
}

div.sphinxsidebarwrapper > ul.current {
    margin-top: 0;
}

div.sphinxsidebarwrapper h1.logo {
    margin-top: 0;
    overflow: hidden;
    font-size: 16px;
}

div.sphinxsidebarwrapper h1.logo a {
    margin-left: -3.75em;
    border-bottom: 0;
}

div.sphinxsidebarwrapper h1.logo a:after {
    content: ' Home';
}

div.body li {
    line-height: 1.5em;
}

div.body p {
    line-height: 1.5em;
}

body {
    font-family: 'Overpass', sans-serif;
    font-size: 16px;
    line-height: 1.5em;
    text-rendering: optimizeLegibility;
    -webkit-font-smoothing: antialiased;
}

a:link {
    text-decoration: none;
}

a:visited {
    text-decoration: none;
}

a:hover {
    text-decoration: none;
}

a:active {
    text-decoration: none;
}

pre, tt, code {
    font-family: 'Roboto Mono', 'Consolas', 'Menlo', 'Deja Vu Sans Mono', 'Bitstream Vera Sans Mono', monospace;
}

div.document {
  margin-top: 5rem;
}

/* Topbar */

.full-w,
.full-wh {
  width: 100%;
}

.full-h,
.full-wh {
  height: 100%;
}

.full-dim {
  position: absolute;
  top: 0;
  right: 0;
  bottom: 0;
  left: 0;
}

.icon {
  position: relative;
  display: block;
  width: 1em;
  height: 1em;
}

.icon svg {
  width: 100%;
  height: 100%;
  transition: transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
}

.flex-c {
  display: -webkit-box;
  display: -ms-flexbox;
  display: flex;
}

.flex-v {
  -webkit-box-orient: vertical;
  -webkit-box-direction: normal;
      -ms-flex-direction: column;
          flex-direction: column;
}

.flex-jc-sb {
  -webkit-box-pack: justify;
      -ms-flex-pack: justify;
          justify-content: space-between;
}

.flex-jc-c {
  -webkit-box-pack: center;
      -ms-flex-pack: center;
          justify-content: center;
}

.flex-jc-e {
  -webkit-box-pack: end;
      -ms-flex-pack: end;
          justify-content: flex-end;
}

.flex-w-w {
  -ms-flex-wrap: wrap;
      flex-wrap: wrap;
}

.flex-a-s {
  -webkit-box-align: stretch;
      -ms-flex-align: stretch;
          align-items: stretch;
}

.flex-a-c {
  -webkit-box-align: center;
      -ms-flex-align: center;
          align-items: center;
}

.flex-g-1 {
  -webkit-box-flex: 1;
      -ms-flex: 1;
          flex: 1;
}

.no-list-style,
.no-list-style > li {
  margin: 0;
  padding: 0;
  list-style: none;
}

.wrap {
  margin-left: 1rem;
  margin-right: 1rem;
}

/*
 * -----------------------------------------------------------------------------
 * Media Queries
 * -----------------------------------------------------------------------------
 */

/* 640px */
@media screen and (min-width: 40em) {
  .wrap {
    margin-left: 8.3334%; /* 1/12 */
    margin-right: 8.3334%;
  }

  .video {
    width: 90%;
  }
}

/* 960px */
@media screen and (min-width: 60em) {
  .wrap {
    margin-left: 11.1112%; /* 1/9 */
    margin-right: 11.1112%;
  }
}

/* 1280px */
@media screen and (min-width: 80em) {
  .wrap {
    margin-left: auto;
    margin-right: auto;
    width: 940px; /* Matches the Alibaster theme */
  }
}

.top-bar {
  position: fixed;
  top: 0;
  right: 0;
  left: 0;
  width: 100%;
  height: 3rem;
  font-size: 1.25rem;
  z-index: 2;
  color: #666;
  background: rgba(255, 255, 255, 0.9);
  -webkit-box-shadow: 0 2px 0 0 rgba(0, 0, 0, 0.05);
          box-shadow: 0 2px 0 0 rgba(0, 0, 0, 0.05);
  line-height: 3rem;
  -webkit-transform: translateZ(0);
          transform: translateZ(0);
}
.top-bar,
.top-bar *,
.top-bar *::before,
.top-bar *::after {
  box-sizing: border-box;
}
.top-bar .top-bar-wrapper {
  -webkit-transition: margin 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: margin 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: margin 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar nav li a {
  color: #666;
}
.top-bar nav li a:hover, .top-bar nav li a:focus {
  color: #000;
}
.top-bar .branding:active, .top-bar nav li a:active {
  color: #0f5d92;
  background: rgba(15, 93, 146, 0.2);
  -webkit-box-shadow: 0 2px 0 0 #0f5d92;
          box-shadow: 0 2px 0 0 #0f5d92;
}
.top-bar .branding:active::after, .top-bar nav li a:active::after {
  opacity: 0 !important;
}
.top-bar .branding:focus, .top-bar nav li a:focus {
  color: #000;
  background: rgba(0, 0, 0, 0.1);
  -webkit-box-shadow: 0 2px 0 0 #000;
          box-shadow: 0 2px 0 0 #000;
  border-radius: 0;
}
.top-bar .branding:focus::after, .top-bar nav li a:focus::after {
  opacity: 0 !important;
}
.top-bar .branding:focus.is-active, .top-bar nav li a:focus.is-active {
  background: rgba(15, 93, 146, 0.2);
}
.top-bar .branding {
  position: relative;
  margin-right: 0.5rem;
  font-weight: bold;
  color: #000;
  -webkit-box-shadow: none;
          box-shadow: none;
  font-size: 1.375rem;
}
.top-bar .branding.is-active {
  -webkit-box-shadow: 0 2px 0 0 #0f5d92;
          box-shadow: 0 2px 0 0 #0f5d92;
}
.top-bar .branding .higlass {
  position: relative;
  -webkit-transition: color 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: color 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: color 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar .branding .higlass::after {
  position: absolute;
  display: block;
  content: "";
  bottom: -2px;
  left: 0;
  right: 0;
  height: 2px;
  background: #000;
  -webkit-transform: scale(0, 1);
      -ms-transform: scale(0, 1);
          transform: scale(0, 1);
  opacity: 0;
  -webkit-transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1), opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1), opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1), opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar .branding .higlass:hover::after {
  opacity: 1;
  -webkit-transform: scale(1, 1);
      -ms-transform: scale(1, 1);
          transform: scale(1, 1);
}
.top-bar .branding .icon {
  width: 2rem;
  height: 2rem;
  margin-right: 0.25em;
}
.top-bar .branding .icon svg {
  -webkit-transition: color 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: color 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: color 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar .branding .icon svg .lense, .top-bar .branding .icon svg .inception {
  -webkit-transform-origin: 47% 47%;
      -ms-transform-origin: 47% 47%;
          transform-origin: 47% 47%;
}
.top-bar .branding .icon svg .lense {
  fill: currentColor;
  -webkit-transition: -webkit-transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: -webkit-transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar .branding .icon svg .inception {
  opacity: 0;
  fill: white;
  -webkit-transform: scale(0.16667);
      -ms-transform: scale(0.16667);
          transform: scale(0.16667);
  -webkit-transition: opacity 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0.15s, -webkit-transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0s;
  transition: opacity 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0.15s, -webkit-transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0s;
  -o-transition: transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0s, opacity 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0.15s;
  transition: transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0s, opacity 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0.15s;
  transition: transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0s, opacity 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0.15s, -webkit-transform 0.6s cubic-bezier(0.25, 0.1, 0.25, 1) 0s;
}
.top-bar .branding:hover .higlass::after, .top-bar .branding:active .higlass::after, .top-bar .branding:focus .higlass::after, .top-bar .branding.is-active .higlass::after {
  opacity: 1;
  -webkit-transform: scale(1, 1);
      -ms-transform: scale(1, 1);
          transform: scale(1, 1);
}
.top-bar .branding:hover .higlass-hi, .top-bar .branding:active .higlass-hi, .top-bar .branding:focus .higlass-hi, .top-bar .branding.is-active .higlass-hi {
  color: #0f5d92;
}
.top-bar .branding:hover svg, .top-bar .branding:active svg, .top-bar .branding:focus svg, .top-bar .branding.is-active svg {
  color: #0f5d92;
}
.top-bar .branding:hover svg .lense, .top-bar .branding:active svg .lense, .top-bar .branding:focus svg .lense, .top-bar .branding.is-active svg .lense {
  -webkit-transform: scale(6);
      -ms-transform: scale(6);
          transform: scale(6);
}
.top-bar .branding:hover svg .inception, .top-bar .branding:active svg .inception, .top-bar .branding:focus svg .inception, .top-bar .branding.is-active svg .inception {
  opacity: 1;
  -webkit-transform: scale(1);
      -ms-transform: scale(1);
          transform: scale(1);
}
.top-bar .btn, .top-bar .button-icon {
  margin: 0.75rem 0.25rem;
  padding: 0;
  height: 1.5rem;
  color: #444;
  font-size: 0.8rem;
  line-height: 1.5rem;
  border: 0;
  border-radius: 0.25rem;
  -webkit-box-shadow: 0 0 0 1px #ddd;
          box-shadow: 0 0 0 1px #ddd;
  background: none;
  -webkit-transition: all 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: all 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: all 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar .btn:hover, .top-bar .button-icon:hover, .top-bar .btn:focus, .top-bar .button-icon:focus {
  color: #fff;
  background: #000;
  -webkit-box-shadow: 0 0 0 1px #000;
          box-shadow: 0 0 0 1px #000;
}
.top-bar .btn.is-active, .top-bar .button-icon.is-active, .top-bar .btn:active, .top-bar .button-icon:active {
  color: #fff;
  background: #0f5d92;
  -webkit-box-shadow: 0 0 0 1px #0f5d92;
          box-shadow: 0 0 0 1px #0f5d92;
}
.top-bar .btn.icon-only, .top-bar .button-icon.icon-only, .top-bar .btn.button-icon-only, .top-bar .button-icon.button-icon-only {
  width: 1.5rem;
}
.top-bar .btn.icon-only, .top-bar .button-icon.icon-only {
  padding: 0.25rem;
}
.top-bar .btn .icon, .top-bar .button-icon .icon {
  width: 1rem;
  height: 1rem;
}
.top-bar .btn .icon-info, .top-bar .button-icon .icon-info {
  width: 1.25rem;
  height: 1.25rem;
}
.top-bar nav li {
  line-height: inherit;
}
.top-bar nav li.separated-left::before {
  content: "";
  width: 2px;
  margin: 0.25rem 0.25rem;
  background: #eee;
}
.top-bar nav li > a, .top-bar nav li > button, .top-bar nav li button.is-primary-nav {
  margin: 0 0.125rem;
  padding: 0 0.125rem;
  font-size: 0.85em;
  position: relative;
}
.top-bar nav li > a::after, .top-bar nav li > button::after, .top-bar nav li button.is-primary-nav::after {
  position: absolute;
  display: block;
  content: "";
  bottom: -2px;
  left: 0;
  right: 0;
  height: 2px;
  background: #000;
  -webkit-transform: scale(0, 1);
      -ms-transform: scale(0, 1);
          transform: scale(0, 1);
  opacity: 0;
  -webkit-transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
  -o-transition: transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1), opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1), opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
  transition: transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1), opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.2s cubic-bezier(0.25, 0.1, 0.25, 1);
}
.top-bar nav li > a:hover::after, .top-bar nav li > button:hover::after, .top-bar nav li button.is-primary-nav:hover::after {
  opacity: 1;
  -webkit-transform: scale(1, 1);
      -ms-transform: scale(1, 1);
          transform: scale(1, 1);
}
.top-bar nav li > a.is-active, .top-bar nav li > button.is-active, .top-bar nav li button.is-primary-nav.is-active {
  font-weight: bold;
}
.top-bar nav li > a.is-active::before, .top-bar nav li > button.is-active::before, .top-bar nav li button.is-primary-nav.is-active::before {
  display: none;
}
.top-bar nav li > a {
  position: relative;
  display: block;
  -webkit-box-shadow: none;
          box-shadow: none;
}
.top-bar nav li > a.icon-only::after {
  display: none;
}
.top-bar nav li > a.icon-only.flex-c {
  display: -webkit-box;
  display: -ms-flexbox;
  display: flex;
}
.top-bar nav li > a.is-active {
  color: #0f5d92;
  -webkit-box-shadow: 0 2px 0 0 #0f5d92;
          box-shadow: 0 2px 0 0 #0f5d92;
}
.top-bar nav li > button, .top-bar nav li button.is-primary-nav {
  height: 3rem;
  border: 0;
  border-radius: 0;
}
.top-bar nav li > button:hover, .top-bar nav li button.is-primary-nav:hover {
  color: #000;
}
.top-bar nav li > button:focus, .top-bar nav li button.is-primary-nav:focus {
  outline: none;
}
.top-bar nav li > button:last-child, .top-bar nav li button.is-primary-nav:last-child {
  margin-right: 0;
}
.top-bar nav li > button.is-active, .top-bar nav li button.is-primary-nav.is-active {
  color: #000;
  -webkit-box-shadow: 0 2px 0 0 #000;
          box-shadow: 0 2px 0 0 #000;
  background: rgba(0, 0, 0, 0.07);
}
.top-bar nav li .icon {
  width: 1em;
  height: 1em;
}
.top-bar .hamburger-wrapper {
  position: absolute;
  top: 0;
  right: 0;
  padding: 0 1rem;
}
.top-bar .text-only {
  height: auto;
}
@media screen and (max-width: 40em) {
  .top-bar .fullscreen-launch-hint {
    display: none;
  }
}
@media screen and (max-width: 30em) {
  .top-bar .branding-launch {
    position: relative;
    z-index: 1;
  }
  .top-bar nav {
    display: block;
    position: absolute;
    top: 3rem;
    right: 0;
    left: 0;
  }
  .top-bar nav .hamburger-wrapper {
    top: -3rem;
  }
  .top-bar nav ul {
    height: 0;
    opacity: 0;
    background: rgba(234, 234, 234, 0.9);
    -webkit-transform: scale(1, 0);
        -ms-transform: scale(1, 0);
            transform: scale(1, 0);
    -webkit-transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
    transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
    -o-transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
    transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
    transition: opacity 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1), -webkit-transform 0.15s cubic-bezier(0.25, 0.1, 0.25, 1);
    -webkit-transform-origin: top center;
        -ms-transform-origin: top center;
            transform-origin: top center;
  }
  .top-bar nav ul.flex-c {
    display: block;
  }
  .top-bar nav li {
    text-align: center;
  }
  .top-bar nav li a {
    height: 2em;
    margin: 0 1rem;
    font-size: 1em;
    line-height: 2em;
  }
  .top-bar nav li a.icon-only {
    width: 100%;
  }
  .top-bar nav li a.icon-only::after {
    display: block;
  }
  .top-bar nav li.separated-left::before {
    display: none;
  }
  .top-bar nav li.separated-left > a {
    -webkit-box-flex: 1;
        -ms-flex-positive: 1;
            flex-grow: 1;
  }
  .top-bar nav li.separated-left > a .flex-a-c {
    -webkit-box-pack: center;
        -ms-flex-pack: center;
            justify-content: center;
  }
  .top-bar nav.is-shown ul {
    height: auto;
    opacity: 1;
    -webkit-transform: scale(1, 1);
        -ms-transform: scale(1, 1);
            transform: scale(1, 1);
  }
}
@media screen and (min-width: 30em) {
  .top-bar .hamburger {
    display: none;
  }
}
